import create from "zustand";
import produce from "immer";
import * as THREE from "three";
export default create(function (set, get) { return ({
    windows: {},
    addWindow: function (window, id) {
        var uuid = id !== null && id !== void 0 ? id : THREE.MathUtils.generateUUID();
        set(produce(function (state) {
            state.windows[uuid] = {
                element: window,
            };
        }));
        return uuid;
    },
    removeWindow: function (id) {
        return set(produce(function (state) {
            delete state.windows[id];
        }));
    },
    addScene: function (scene, id, camera) {
        return set(produce(function (state) {
            if (state.windows[id]) {
                var elem = state.windows[id].element;
                var rect = elem.getBoundingClientRect();
                state.windows[id].scene = scene;
                state.windows[id].camera =
                    camera !== null && camera !== void 0 ? camera : new THREE.PerspectiveCamera(75, rect.width / rect.height, 0.1, 1000);
                state.windows[id].hasInit = false;
            }
        }));
    },
    removeScene: function (id) { return get().removeWindow(id); },
    sethasInit: function (hasInit, id) {
        return set(produce(function (state) {
            state.windows[id].hasInit = hasInit;
        }));
    },
    getIds: function () { return Object.keys(get().windows); },
    frameSubscribers: {},
    addSubscriber: function (cb, uuid) {
        return set(produce(function (state) {
            if (uuid.length > 0) {
                uuid.forEach(function (id) { return (state.frameSubscribers[id] = cb); });
            }
            else {
                Object.keys(state.windows).forEach(function (k) { return (state.frameSubscribers[k] = cb); });
            }
        }));
    },
    removeSubscriber: function (uuid) {
        return set(produce(function (state) {
            if (uuid.length > 0) {
                uuid.forEach(function (id) { return delete state.frameSubscribers[id]; });
            }
            else {
                state.frameSubscribers = {};
            }
        }));
    },
    initSubscribers: {},
    addInitSubscriber: function (cb, uuid) {
        return set(produce(function (state) {
            if (uuid.length > 0) {
                uuid.forEach(function (id) { return (state.initSubscribers[id] = cb); });
            }
            else {
                Object.keys(state.windows).forEach(function (k) { return (state.initSubscribers[k] = cb); });
            }
        }));
    },
    removeInitSubscriber: function (uuid) {
        return set(produce(function (state) {
            if (uuid.length > 0) {
                uuid.forEach(function (id) { return delete state.initSubscribers[id]; });
            }
            else {
                state.initSubscribers = {};
            }
        }));
    },
}); });
//# sourceMappingURL=store.js.map